"""
@Author：WangYuXiang
@E-mile：Hill@3io.cc
@CreateTime：2021/1/29 16:38
@DependencyLibrary：无
@MainFunction：无
@FileDoc： 
    utils.py
    文件说明
@ChangeHistory:
    datetime action why
    example:
    2021/1/29 16:38 change 'Fix bug'
        
"""
import datetime


class TestDataMixin:
    min_int = 1
    max_int = 9999
    min_float = 0.01
    max_float = 9999.99
    pi = 3.1415926
    str_min_int = '1'
    str_max_int = '9999'
    str_min_float = '0.01'
    str_max_float = '9999.99'
    str_pi = '3.1415926'
    str_chinese_w = ' 测试字符 '
    str_chinese = '测试字符'
    str_england = 'Test String'
    str_england_w = ' Test String '
    bool_True = True
    bool_False = False
    str_bool_True = 'True'
    str_bool_False = 'False'
    long_str = '996' * 600
    str_datetime = '2019-12-18 08:21:25'
    str_datetime_bad_y = '99999-12-18 08:21:25'
    str_datetime_bad_ym = '2019-13-18 08:21:25'
    str_datetime_bad_d = '2019-12-32 08:21:25'
    str_datetime_bad_h = '2019-12-18 25:21:25'
    str_datetime_bad_hm = '2019-12-18 08:61:25'
    str_datetime_bad_s = '2019-12-18 08:21:61'
    str_date = '2019-12-18'
    str_date_bad_y = '99999-12-18'
    str_date_bad_m = '2019-13-18'
    str_date_bad_d = '2019-12-32'
    str_time = '08:21:25'
    str_time_bad_h = '25:59:25'
    str_time_bad_m = '08:61:25'
    str_time_bad_s = '08:59:61'
    obj_datetime = datetime.datetime(2019, 12, 18, 8, 21, 25)
    obj_date = datetime.date(2019, 12, 18)
    obj_time = datetime.time(8, 21, 25)



"""
@Author：WangYuXiang
@E-mile：Hill@3io.cc
@CreateTime：2021/1/29 10:23
@DependencyLibrary：无
@MainFunction：无
@FileDoc：
    test_integer_field.py
    测试整数类型字段
@ChangeHistory:
    datetime action why
    example:
    2021/1/29 10:23 change 'Fix bug'

"""

import unittest

from sanic_rest_framework.exceptions import ValidationError
from sanic_rest_framework.fields import IntegerField as TestField
from sanic_rest_framework.test.test_fields.test_base_field import TestBaseField


class TestCharField(TestBaseField):

    def test_external_to_internal(self):
        """
        外转内 str -> dict 是严格的，不符合类型的都应该报错,
        一切都要经过验证
        int => [1,'1','1.0',1.0',1.]
        :return:
        """

        tf = TestField()
        with self.assertRaises(ValidationError):
            tf.external_to_internal(self.max_float)
        with self.assertRaises(ValidationError):
            tf.external_to_internal(self.str_max_float)
        with self.assertRaises(ValidationError):
            tf.external_to_internal(self.str_chinese)
        with self.assertRaises(ValidationError):
            tf.external_to_internal(self.bool_True)

        self.assertEqual(tf.external_to_internal(self.max_int), self.max_int)
        self.assertEqual(tf.external_to_internal(self.str_max_int), self.max_int)

    async def test_internal_to_external(self):
        """
        内转外 str -> dict 是宽松的，
        只要是数值类型都不报错，int(xx)
        :return:
        """
        data1 = {'tf1': self.str_chinese}
        data2 = {'tf1': self.max_int}
        data3 = {'tf1': self.str_max_int}
        data4 = {'tf1': self.str_max_float}
        data5 = {'tf1': self.max_float}

        tf = TestField()
        tf.bind('tf1', tf)

        value = await tf.get_internal_value(data1)
        with self.assertRaises(ValueError):
            await tf.internal_to_external(value)

        value = await tf.get_internal_value(data2)
        self.assertEqual(await tf.internal_to_external(value), self.max_int)

        value = await tf.get_internal_value(data3)
        self.assertEqual(await tf.internal_to_external(value), self.max_int)
        value = await tf.get_internal_value(data4)

        with self.assertRaises(ValueError):
            # str_float 不可被转换
            await tf.internal_to_external(value)

        value = await tf.get_internal_value(data5)
        self.assertEqual(await tf.internal_to_external(value), self.max_int)

    def test_max_value(self):
        tf1 = TestField()
        tf2 = TestField(max_value=10)

        self.assertEqual(tf1.run_validators(data=self.str_max_int), None)

        # 超出限制
        with self.assertRaises(ValidationError):
            self.assertEqual(tf2.run_validators(data=self.str_max_int), None)

        # 未设置 不存在超出限制
        self.assertEqual(tf1.run_validators(data=self.max_int), None)
        self.assertEqual(tf1.run_validators(data=self.min_int), None)

        # 超出限制
        with self.assertRaises(ValidationError):
            self.assertEqual(tf2.run_validators(data=self.max_int), None)
        self.assertEqual(tf2.run_validators(data=self.min_int), None)

    def test_min_value(self):
        tf1 = TestField()
        tf2 = TestField(min_value=10)

        # 无限制，则无验证器所以不报错
        self.assertEqual(tf1.run_validators(data=self.str_max_int), None)

        with self.assertRaises(ValidationError):
            self.assertEqual(tf2.run_validators(data=self.str_max_int), None)
            # 超出限制

        # 未设置 不存在超出限制
        self.assertEqual(tf1.run_validators(data=self.max_int), None)
        self.assertEqual(tf1.run_validators(data=self.min_int), None)

        # 低于最小值
        with self.assertRaises(ValidationError):
            self.assertEqual(tf2.run_validators(data=self.min_int), None)
        self.assertEqual(tf2.run_validators(data=self.max_int), None)

    def test_max_string_length(self):
        tf1 = TestField()
        with self.assertRaises(ValidationError):
            #  超出约定长度
            tf1.external_to_internal(self.long_str)
        self.assertEqual(tf1.external_to_internal(self.str_max_int), self.max_int)


if __name__ == '__main__':
    unittest.main()
